@@ -15,8 +15,8 @@
 import warnings
 from qubell import deprecated
 from qubell.api.private.common import EntityList, IdName
-from qubell.api.private.service import system_application_types, COBALT_SECURE_STORE_TYPE, WORKFLOW_SERVICE_TYPE, \
-    SHARED_INSTANCE_CATALOG_TYPE
+from qubell.api.private.service import system_application_types, system_application_parameters, COBALT_SECURE_STORE_TYPE, WORKFLOW_SERVICE_TYPE, \
+    SHARED_INSTANCE_CATALOG_TYPE, STATIC_RESOURCE_POOL_TYPE
 from qubell.api.tools import lazyproperty
 
 __author__ = "Vasyl Khomenko"
@@ -26,6 +26,7 @@ __email__ = "vkhomenko@qubell.com"
 
 import logging as log
 import simplejson as json
+import copy
 
 from qubell.api.private.manifest import Manifest
 from qubell.api.private import exceptions
@@ -33,18 +34,15 @@ from qubell.api.private.instance import InstanceList, DEAD_STATUS, Instance
 from qubell.api.private.application import ApplicationList
 from qubell.api.private.environment import EnvironmentList
 from qubell.api.private.zone import ZoneList
+from qubell.api.private.provider import ProviderList
 from qubell.api.provider.router import ROUTER as router
+from qubell.api.private.common import QubellEntityList, Entity
 
 
-class Organization(object):
+class Organization(Entity):
 
     def __init__(self, id, auth=None):
-        self.providers = []
-
-        self.organizationId = id
-
-        my = self.json()
-        self.name = my['name']
+        self.organizationId = self.id = id
 
     @staticmethod
     def new(name):
@@ -73,35 +71,62 @@ class Organization(object):
     @lazyproperty
     def zones(self): return ZoneList(self)
 
+    @lazyproperty
+    def providers(self): return ProviderList(self)
+
     @property
     def defaultEnvironment(self): return self.get_default_environment()
 
     @property
     def zone(self): return self.get_default_zone()
 
+    @property
+    def name(self): return self.json()['name']
+
     def json(self):
-        resp = router.get_organizations()
-        org = [x for x in resp.json() if x['id'] == self.organizationId]
-        if len(org)>0:
-            return org[0]
-        return resp.json()
+        return router.get_organization(org_id=self.organizationId).json()
 
     def restore(self, config):
-        for instance in config.pop('instances', []):
-            launched = self.get_or_launch_instance(id=instance.pop('id', None), name=instance.pop('name'), **instance)
-            assert launched.ready()
-        for serv in config.pop('services',[]):
-            self.get_or_create_service(id=serv.pop('id', None), name=serv.pop('name'), type=serv.pop('type', None))
-        for prov in config.get('providers', []):
-            self.get_or_create_provider(id=prov.pop('id', None), name=prov.pop('name'), parameters=prov)
-        for env in config.pop('environments',[]):
-            restored_env = self.get_or_create_environment(id=env.pop('id', None), name=env.pop('name', 'default'),zone=env.pop('zone', None), default=env.pop('default', False))
-            restored_env.clean()
-            restored_env.restore(env)
+        config = copy.deepcopy(config)
+        for prov in config.get('cloudAccounts', []):
+            self.provider(id=prov.pop('id', None),
+                                        name=prov.pop('name'),
+                                        parameters=prov)
+
         for app in config.pop('applications'):
-            mnf = app.pop('manifest', None)
-            restored_app = self.application(id=app.pop('id', None), manifest=Manifest(**mnf), name=app.pop('name'))
-            restored_app.restore(app)
+            manifest_param = {k: v for k, v in app.iteritems() if k in ["content", "url", "file"]}
+            if manifest_param:
+                manifest = Manifest(**manifest_param)
+            else:
+                manifest = None  # if application exists, manifest must be None
+            self.application(id=app.pop('id', None),
+                                            manifest=manifest,
+                                            name=app.pop('name'))
+
+        for serv in config.pop('services',[]):
+            app=serv.pop('application', None)
+            if app:
+                app = self.get_application(name=app)
+            self.get_or_create_service(id=serv.pop('id', None),
+                                       name=serv.pop('name'),
+                                       type=serv.pop('type', None),
+                                       application=app,
+                                       parameters=serv.pop('parameters', None))
+
+        for env in config.pop('environments',[]):
+            restored_env = self.get_or_create_environment(id=env.pop('id', None),
+                                                          name=env.pop('name', 'default'),
+                                                          zone=env.pop('zone', None),
+                                                          default=env.pop('default', False))
+            restored_env.restore(env)
+
+        #todo: make launch and ready async
+        for instance in config.pop('instances', []):
+            launched = self.get_or_launch_instance(application=self.get_application(name=instance.pop('application')),
+                                                   id=instance.pop('id', None),
+                                                   name=instance.pop('name', None),
+                                                   **instance)
+            assert launched.ready()
 
 ### APPLICATION
     def create_application(self, name=None, manifest=None):
@@ -117,8 +142,8 @@ class Organization(object):
     def get_application(self, id=None, name=None):
         """ Get application object by name or id.
         """
-        criteria = id or name
-        return self.applications[criteria]
+        log.info("Picking application: %s" % (id or name))
+        return self.applications[id or name]
 
     def list_applications_json(self):
         """ Return raw json
@@ -185,17 +210,18 @@ class Organization(object):
 
 # INSTANCE
 
-    def create_instance(self, application, revision=None, environment=None, name=None, parameters=None,
+    def create_instance(self, application, revision=None, environment=None, name=None, parameters=None, submodules=None,
                         destroyInterval=None):
         """ Launches instance in application and returns Instance object.
         """
         from qubell.api.private.instance import Instance
-        return Instance.new(application, revision, environment, name, parameters, destroyInterval)
+        return Instance.new(application, revision, environment, name, parameters, submodules, destroyInterval)
 
     def get_instance(self, id=None, name=None):
         """ Get instance object by name or id.
         If application set, search within the application.
         """
+        log.info("Picking instance: %s" % (id or name))
         if id:  # submodule instances are invisible for lists
             return Instance(id=id, organization=self)
         return self.instances[id or name]
@@ -209,7 +235,7 @@ class Organization(object):
             instances = router.get_instances(org_id=self.organizationId).json()
         return [ins for ins in instances if ins['status'] not in DEAD_STATUS]
 
-    def get_or_create_instance(self, id=None, application=None, revision=None, environment=None, name=None, parameters=None,
+    def get_or_create_instance(self, id=None, application=None, revision=None, environment=None, name=None, parameters=None, submodules=None,
                                destroyInterval=None):
         """ Get instance by id or name.
         If not found: create with given parameters
@@ -221,17 +247,17 @@ class Organization(object):
                 instance.ready()
             return instance
         except exceptions.NotFoundError:
-            return self.create_instance(application, revision, environment, name, parameters, destroyInterval)
+            return self.create_instance(application, revision, environment, name, parameters, submodules, destroyInterval)
     get_or_launch_instance = get_or_create_instance
 
-    def instance(self, id=None, application=None, name=None, revision=None, environment=None, parameters=None, destroyInterval=None):
+    def instance(self, id=None, application=None, name=None, revision=None, environment=None, parameters=None, submodules=None, destroyInterval=None):
         """ Smart method. It does everything, to return Instance with given parameters within the application.
         If instance found running and given parameters are actual: return it.
         If instance found, but parameters differs - reconfigure instance with new parameters.
         If instance not found: launch instance with given parameters.
         Return: Instance object.
         """
-        instance = self.get_or_create_instance(id, application, revision, environment, name, parameters, destroyInterval)
+        instance = self.get_or_create_instance(id, application, revision, environment, name, parameters, submodules, destroyInterval)
 
         reconfigure = False
         # if found:
@@ -248,41 +274,28 @@ class Organization(object):
 
 
 ### SERVICE
-    def create_service(self, application, revision=None, environment=None, name=None, parameters=None,
-                       destroyInterval=None):
-        if application.name in system_application_types.values():
-             destroyInterval = -1  # never for system applications
+    def create_service(self, application=None, revision=None, environment=None, name=None, parameters=None, type=None):
 
-        instance = self.create_instance(application, revision, environment, name, parameters, destroyInterval)
+        if type and system_application_types.has_key(type):
+            if application: log.warning('Ignoring application parameter (%s) while creating system service' % application)
+            application = self.applications[system_application_types[type]]
+
+        instance = self.create_instance(application, revision, environment, name, parameters)
         instance.environment.add_service(instance)
         return instance
 
-    def create_keystore_service(self, name='generated-keystore', parameters=None):
-        application = self.applications[system_application_types(COBALT_SECURE_STORE_TYPE)]
-        return self.create_service(name=name, application=application, parameters=parameters)
-
-    def create_workflow_service(self, name='generated-workflow', policies=None, zone=None):
-        parameters = {'configuration.policies': json.dumps(policies or {})}
-        application = self.applications[system_application_types(WORKFLOW_SERVICE_TYPE)]
-        return self.create_service(name=name, application=application, parameters=parameters)
-
-    def create_shared_service(self, name='generated-shared', instances=None, zone=None):
-        parameters = {'configuration.shared-instances': json.dumps(instances or {})}
-        application = self.applications[system_application_types(SHARED_INSTANCE_CATALOG_TYPE)]
-        return self.create_service(name=name, application=application, parameters=parameters)
-
     get_service = get_instance
 
     def list_services_json(self):
         return router.get_services(org_id=self.organizationId).json()
 
     def get_or_create_service(self, id=None, application=None, revision=None, environment=None, name=None, parameters=None,
-                              destroyInterval=None):
+                              type=None, destroyInterval=None):
         """ Get by name or create service with given parameters"""
         try:
             return self.get_instance(id=id, name=name)
         except exceptions.NotFoundError:
-            return self.create_service(application, revision, environment, name, parameters, destroyInterval)
+            return self.create_service(application, revision, environment, name, parameters, type)
 
     service = get_or_create_service
 
@@ -303,8 +316,8 @@ class Organization(object):
     def get_environment(self, id=None, name=None):
         """ Get environment object by name or id.
         """
-        criteria = id or name
-        return self.environments[criteria]
+        log.info("Picking environment: %s" % (id or name))
+        return self.environments[id or name]
 
     def delete_environment(self, id):
         env = self.get_environment(id)
@@ -386,48 +399,39 @@ class Organization(object):
     def create_provider(self, name, parameters):
         log.info("Creating provider: %s" % name)
         parameters['name'] = name
-        resp = router.post_provider(org_id=self.organizationId, data=json.dumps(parameters))
+        resp = router.post_organization_provider(org_id=self.organizationId, data=json.dumps(parameters))
         return self.get_provider(resp.json()['id'])
 
     def list_providers_json(self):
         return router.get_providers(org_id=self.organizationId).json()
 
-    @deprecated("use list_providers_json instead")
-    def list_providers(self): return self.list_providers_json()
-
-    def get_provider(self, id):
-        from qubell.api.private.provider import Provider
-        prov = Provider(organization=self, id=id)
-        self.providers.append(prov)
-        return prov
+    def get_provider(self, id=None, name=None):
+        log.info("Picking provider: %s" % (id or name))
+        return self.providers[id or name]
 
     def delete_provider(self, id):
         prov = self.get_provider(id)
-        self.providers.remove(prov)
         return prov.delete()
 
-    def get_or_create_provider(self,id=None, name=None, parameters=None):
-
-        """ Smart object. Will create provider or pick one, if exists"""
-        if name:
-            provs = [prov for prov in self.list_providers_json() if prov['name'] == name]
-            # provider found by name
-            if len(provs):
-                return self.get_provider(provs[0]['id']) # pick first
-            elif parameters:
-                return self.create_provider(name=name, parameters=parameters)
-        else:
-            name = 'generated-provider'
-            if id:
-                return self.get_provider(id)
-            elif parameters:
-                return self.create_provider(name=name, parameters=parameters)
-        raise exceptions.NotFoundError('Provider not found or not enough parameters to create provider: %s' % name)
+    def get_or_create_provider(self, id=None, name=None, parameters=None):
+        """ Get or create provider
+        """
+        assert id or name
+        try:
+            return self.get_provider(id=id, name=name)
+        except exceptions.NotFoundError:
+            return self.create_provider(name, parameters)
 
     def provider(self, id=None, name=None, parameters=None):
-        """ Get , create or modify provider
+        """ (Get and set) or create provider
         """
-        return self.get_or_create_provider(id=id, name=name, parameters=parameters)
+        assert id or name
+        try:
+            provider = self.get_provider(id=id, name=name)
+            provider.update(name=name or provider.name, parameters=parameters)
+            return provider
+        except exceptions.NotFoundError:
+            return self.create_provider(name, parameters)
 
 ### ZONES
 
@@ -437,33 +441,27 @@ class Organization(object):
     def get_zone(self, id=None, name=None):
         """ Get zone object by name or id.
         """
-        if id:
-            zones = [x for x in self.zones if x.id == id]
-        elif name:
-            zones = [x for x in self.zones if x.name == name]
-        else:
-            raise exceptions.NotEnoughParams('No name nor id given. Unable to get application')
-        if len(zones) == 1:
-            return zones[0]
-        elif len(zones) > 1:
-            log.warning('Found several zones with name %s. Picking last' % name)
-            return zones[-1]
-        raise exceptions.NotFoundError('Unable to get zone by id: %s' % id)
+        log.info("Picking zone: %s" % (id or name))
+        return self.zones[id or name]
 
     def get_default_zone(self):
-    # Zones(backends) are factor we can't controll. So, get them.
-        backends = self.json()['backends']
-        zones = [bk for bk in backends if bk['isDefault']==True]
+        backends = self.get_default_environment().json()['backends']
+        zones = [bk for bk in backends if bk['isDefault'] == True]
         if len(zones):
             zoneId = zones[0]['id']
             return self.get_zone(id=zoneId)
         raise exceptions.NotFoundError('Unable to get default zone')
 
-class OrganizationList(EntityList):
+
+class OrganizationList(QubellEntityList):
+    base_clz = Organization
+
+    """
     def __init__(self, list_json_method):
         self.json = list_json_method
         EntityList.__init__(self)
     def _id_name_list(self):
         self._list = [IdName(ent['id'], ent['name']) for ent in self.json()]
     def _get_item(self, id_name):
-        return Organization(id=id_name.id)
\ No newline at end of file
+        return Organization(id=id_name.id)
+    """
