@@ -12,7 +12,7 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from qubell.api.public import application
+
 
 __author__ = "Anton Panasenko"
 __copyright__ = "Copyright 2013, Qubell.com"
@@ -23,23 +23,33 @@ __email__ = "apanasenko@qubell.com"
 import unittest
 import yaml
 import logging as log
+import re
 
 from functools import wraps
 
-from qubell.api.private.instance import Instance
 from qubell.api.private.manifest import Manifest
 from qubell.api.private.service import system_application_types, COBALT_SECURE_STORE_TYPE, WORKFLOW_SERVICE_TYPE
 
-
-from requests import api
-from requests import sessions
-
-import time
 import logging
-import re
 import types
+
 logging.getLogger("requests.packages.urllib3.connectionpool").setLevel(logging.WARN)
 
+def format_as_api(data):
+    """
+    Accepts {'default':{},}
+    returns [{'name':'default',}]
+    """
+    result = []
+    if isinstance(data, dict):
+        for name, value in data.items():
+            key = str(re.sub("[^a-zA-Z0-9_]", "", name))
+            value.update({'name': name})
+            result.append(value)
+        return result
+    else:
+        return data
+
 def values(names):
     """
     Method decorator that allows inject return values into method parameters.
@@ -99,13 +109,13 @@ def workflow(name, parameters=None, timeout=10):
     return wrapper
 
 
-def environment(params):
+def environment(envdata):
     """
     Class decorator that allows to run tests in sandbox against different Qubell environments.
     Each test method in suite is converted to <test_name>_on_environemnt_<environment_name>
     :param params: dict
     """
-    assert isinstance(params, dict), "@environment decorator should take 'dict' with environments"
+    #assert isinstance(params, dict), "@environment decorator should take 'dict' with environments"
 
     def copy(func, name=None):
         return types.FunctionType(func.func_code, func.func_globals, name=name,
@@ -114,19 +124,22 @@ def environment(params):
 
     def wraps_class(clazz):
         if "environments" in clazz.__dict__:
-            log.warn("Class {0} environment attribute is overriden".format(clazz.__name__))
-
+            log.warn("Class {0} environment attribute is overridden".format(clazz.__name__))
+        params = format_as_api(envdata)
         clazz.environments = params
 
         methods = [method
                    for _, method in clazz.__dict__.items()
                    if isinstance(method, types.FunctionType) and method.func_name.startswith("test") ]
+        for env in params:
+            if env['name'] != 'default':
+                env['name'] += '_for_%s' % clazz.__name__   # Each test class should have it's own set of envs.
 
         for method in methods:
             delattr(clazz, method.func_name)
             log.info("Test '{0}' multiplied per environment in {1}".format(method.func_name, clazz.__name__))
-            for env_name in params.keys():
-                new_name = method.func_name + "_on_environment_" + env_name
+            for env in params:
+                new_name = method.func_name + "_on_environment_" + env['name']
                 setattr(clazz, new_name, copy(method, new_name))
 
         return clazz
@@ -153,7 +166,7 @@ def instance(byApplication):
 
             def find_by_application_name(app):
                 for inst in self.instances:
-                    if inst.applicationName == app and inst.environmentName == env:
+                    if inst.application.name == app and inst.environment.name == env:
                         return inst
                 return None
 
@@ -170,16 +183,17 @@ class BaseTestCase(unittest.TestCase):
 
     @classmethod
     def environment(cls, organization):
-        import re
-        if cls.environments:
-            envs = {}
-            servs = []
-            for name, value in cls.environments.items():
-                key = str(re.sub("[^a-zA-Z0-9_]", "", name))
-                envs.update({key: value})
 
-        else:
-            envs = {"default": {}}
+        addon = {"provider": {"name": cls.parameters['provider_name']},
+                 "services":
+                    [{"name": "Default credentials service"},
+                     {"name": "Default workflow service"}
+                    ]}
+
+        envs = cls.environments or [{"name": "default"},]
+
+        for env in envs:
+            env.update(addon) # Add provider, keystore, workflow to every env.
 
         servs = [{"type": COBALT_SECURE_STORE_TYPE, "name": 'Default credentials service'},
                  {"type": WORKFLOW_SERVICE_TYPE, "name": 'Default workflow service', "parameters": {'configuration.policies': '{}'}}]
@@ -200,10 +214,6 @@ class BaseTestCase(unittest.TestCase):
             }],
             "environments": envs}
 
-    @classmethod
-    def apps(cls):
-        return []
-
     @classmethod
     def timeout(cls):
         return 15
@@ -211,67 +221,84 @@ class BaseTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         super(BaseTestCase, cls).setUpClass()
+
         if cls.parameters['organization']:
             cls.prepare(cls.parameters['organization'], cls.timeout())
         else:
             cls.prepare(cls.__name__, cls.timeout())
 
-    @classmethod
-    def prepare(cls, organization, timeout=30):
-        cls.sandbox = SandBox(cls.platform, cls.environment(organization))
-        cls.organization = cls.sandbox.make()
-        cls.instances = []
-
-        def launch_in_env(app, env):
-            environment = cls.organization.environment(name=env)
-            parameters = {'parameters': app.get('parameters', {})}
-            instance = cls.organization.applications[app['name']].launch(
-                environment=environment, parameters=parameters)
-            cls.instances.append(instance)
-            if app.get('add_as_service', False):
-                environment.add_service(instance)
-            cls.sandbox.sandbox["instances"].append({
-                "id": instance.instanceId,
-                "name": instance.name,
-                "applicationId":  app['id']
-            })
-            return instance
-
-        def check_instances(instances):
-            for instance in cls.instances:
-                if not instance.ready(timeout=timeout):
-                    cls.sandbox.clean()
-                    assert False, "Instance %s not ready after timeout" % instance.instanceId
-
-        # launch service instances first
-        for app in cls.sandbox['applications']:
-            for env in cls.sandbox['environments']:
-                if app.get('launch', True) and app.get('add_as_service', False):
-                    launch_in_env(app, env)
-        check_instances(cls.instances)
-
-        # then launch non-service instances
-        regular_instances = []
-        for app in cls.sandbox['applications']:
-            for env in cls.sandbox['environments']:
-                if app.get('launch', True) and not app.get('add_as_service', False):
-                    regular_instances.append(launch_in_env(app, env))
-        check_instances(regular_instances)
-
     @classmethod
     def tearDownClass(cls):
         super(BaseTestCase, cls).tearDownClass()
         cls.clean()
 
     @classmethod
-    def clean(cls):
-        if cls.sandbox:
-            cls.sandbox.clean()
+    def prepare(cls, organization, timeout=30):
+        """ Create sandboxed test environment
+        """
+        cls.sandbox = SandBox(cls.platform, cls.environment(organization))
+        cls.organization = cls.sandbox.make()
+        cls.instances = cls.organization.instances
+
+        def launch_in_env(app, env):
+            environment = cls.organization.environments[env['name']]
+            application = cls.organization.applications[app['name']]
+            parameters = app.get('parameters', {})
+            instance = cls.organization.create_instance(application=application,
+                                                        environment=environment,
+                                                        parameters=parameters)
+            if app.get('add_as_service', False):
+                environment.add_service(instance)
+            cls.sandbox.sandbox["instances"].append({
+                "id": instance.instanceId,
+                "name": instance.name,
+            })
+            return instance
+
+        def check_instances(instances):
+            for instance in instances:
+                if not instance.ready(timeout=timeout):
+                    error = instance.error.strip()
+
+                    # TODO: if instance fails to start during tests, add proper unittest log
+                    cls.clean()
+                    assert not error, "Instance %s didn't launch properly and has error '%s'" % (instance.instanceId, error)
+                    assert False, "Instance %s is not ready after %s minutes and stop on timeout" % (instance.instanceId, timeout)
+
+        # launch service instances first
+        cls.service_instances = []
+        for app in cls.sandbox['applications']:
+            for env in cls.sandbox['environments']:
+                if app.get('launch', True) and app.get('add_as_service', False):
+                    cls.service_instances.append(launch_in_env(app, env))
+        check_instances(cls.service_instances)
+
+        # then launch non-service instances
+        cls.regular_instances = []
+        for app in cls.sandbox['applications']:
+            for env in cls.sandbox['environments']:
+                if app.get('launch', True) and not app.get('add_as_service', False):
+                    cls.regular_instances.append(launch_in_env(app, env))
+        check_instances(cls.regular_instances)
+
+    @classmethod
+    def clean(cls, timeout=10):
+        def destroy_instances(instances):
+            for instance in instances:
+                instance.destroy()
+                if not instance.destroyed(timeout):
+                    log.error(
+                        "Instance was not destroyed properly {0}: {1}", instance.id, instance.name)
+
+        log.info("Cleaning sandbox...")
+        destroy_instances(cls.regular_instances)
+        destroy_instances(cls.service_instances)
+        log.info("Sandbox cleaned")
 
     # noinspection PyPep8Naming
     def findByApplicationName(self, name):
         for instance in self.instances:
-            if instance.applicationName == name:
+            if instance.application.name == name:
                 return instance
 
 
@@ -280,91 +307,22 @@ class SandBox(object):
         self.sandbox = sandbox
         self.platform = platform
         self.organization = self.platform.organization(name=self.sandbox["organization"]["name"])
+        self.sandbox['instances'] = sandbox.get('instances', [])
+
 
     @staticmethod
     def load_yaml(platform, yaml_file):
         return SandBox(platform, yaml.safe_load(yaml_file))
 
-    def __service(self, environment, service_data):
-
-        application = self.organization.applications[system_application_types.get(service_data["type"],service_data["type"])]
-        service = self.organization.service(application=application, name=service_data["name"],
-                                            environment=environment,
-                                            parameters=(service_data.get("parameters")))
-        service.ready(1)
-        environment.add_service(service)
-        if COBALT_SECURE_STORE_TYPE is service_data["type"]:
-            key_id = service.regenerate()['id']
-            environment.add_policy({
-                "action": "provisionVms",
-                "parameter": "publicKeyId",
-                "value": key_id
-            })
-        service_data["id"] = service.instanceId
-
-    def __cloud_account(self, environment, provider):
-        cloud = self.organization.provider(parameters=provider, name=provider["name"])
-        environment.add_provider(cloud)
-        provider["id"] = cloud.providerId
-
-    def __application(self, app):
-        manifest = Manifest(**{k: v for k, v in app.iteritems() if k in ["content", "url", "file"]})
-        application = self.organization.application(manifest=manifest, name=app["name"])
-        app["id"] = application.applicationId
-
     def make(self):
         log.info("Preparing sandbox...")
-
-        for app in self.sandbox["applications"]:
-            self.__application(app)
-
-        for name, env in self.sandbox["environments"].items():
-            environment = self.organization.environment(name=name)
-            environment.clean()
-
-            for service in self.sandbox["services"] + env.get('services', []):
-                self.__service(environment, service)
-
-            for provider in self.sandbox["cloudAccounts"] + env.get('cloudAccounts', []):
-                self.__cloud_account(environment, provider)
-
-            for police in env.get('policies', []):
-                environment.add_policy(police)
-
+        self.organization.restore(self.sandbox)
         log.info("Sandbox prepared")
-
         return self.organization
 
-    def clean(self, timeout=10):
-        log.info("Cleaning sandbox...")
-
-        def destroy(test_instances):
-            instances = self.organization.instances
-            services_to_destroy = []
-            instances_to_destroy = []
-            for instanceData in test_instances:
-                if instanceData['id'] in instances: #someone may removed it already
-                    instance = instances[instanceData['id']]
-                    if instance.serve_environments:
-                        services_to_destroy.append(instance)
-                    else:
-                        instances_to_destroy.append(instance)
-
-            # destroy non-service instances first
-            for instance in instances_to_destroy:
-                instance.destroy()
-            for instance in services_to_destroy:
-                instance.destroy()
-
-            return services_to_destroy + instances_to_destroy
-
-        for instance in destroy(self.sandbox['instances']):
-            if not instance.destroyed(timeout):
-                log.error(
-                    "Instance was not destroyed properly {0}: {1}", instance.id, instance.name
-                )
-
-        log.info("Sandbox cleaned")
+    def clean(self):
+        # TODO: need cleaning mechanism
+        pass
 
     def __check_environment_name(self, name):
         import re
@@ -374,4 +332,4 @@ class SandBox(object):
         if name in self.sandbox:
             return self.sandbox[name]
         else:
-            return None
\ No newline at end of file
+            return None
